function xdot = gtmnonlin(t,u,x,Aero)
% This function computes the state derivatives for a 4-state
% longitudinal model of the GTM.  This model uses the GTM
% look-up tables for the aerodynamic coefficients.
% 
% States:
%  V = x(1);                   % True Air Speed, V (m/sec); 
%  alpha = x(2);               % Angle of Attack, alpha (rad); 
%  q = x(3);                   % Pitch Rate, q (rad/sec);
%  theta = x(4);               % Pitch Euler Angle, theta (rad)
%
% Inputs:
%  dthr = u(1);                % Throttle Position, dthr (Percent, 0 to 100)
%  de = u(2);                  % Elevator Deflection, de (rad)
%
% 8/5/2009 PJS

% ----- Constants
% Conversion factors
d2r       = pi/180;         % Degrees to Radians [ rad/deg ]
r2d       = 180/pi;         % Radians to Degrees [ deg/rad ]

% Physical Parameters / Constant for EoM
g         = 9.8067;       % Gravitational accel [m/sec^2]
rho0      = 1.2240;       % Air density [ kg/m^3 ]
Sref      = 0.5483;       % Wing reference area [ m^2 ]
mass      = 22.498;       % Mass [kg]
cbar      = 0.2790;       % Mean chord [m]
Iyy       = 5.7676;       % Princ. moment of inertia [kg-m^2]
pratio    = 1.0;          % Pressure / Sea Level Pressure [unitless]

% ----- Get states
V = x(1);                   % True Air Speed, V (m/sec); 
alpha = x(2);               % Angle of Attack, alpha (rad); 
q = x(3);                   % Pitch Rate, q (rad/sec);
theta = x(4);               % Pitch Euler Angle, theta (rad)

% ----- Get inputs
dthr = u(1);                % Throttle Position, dthr (Percent, 0 to 100)
de = u(2);                  % Elevator Deflection, de (deg)

% ----- Engine Model
% L and R engines are assumed to have equal setting.
cg_pos0 = [-4.7750  0  -0.9401];
engl_ang = [0 2.1460 1.6830];
engl_pos = [-4.32525  -1.1833  -0.6425];
engr_ang = [0 2.1460 -1.6830];
engr_pos = [-4.32525  1.1833  -0.6425];
deltaLEng = engl_pos - cg_pos0;

% Convert deltaLEng to SI units
ft2m      = 0.3048;         % ft to meters       [ m/ft ]
deltaLEng = deltaLEng*ft2m;

% Direction cosine vector (DCV) to transform thrust axis 
% to airplane body axis. 
DCV_LEng = [cosd(engl_ang(2))*cosd(engl_ang(3));...
           -sind(engl_ang(3));...
            sind(engl_ang(2))*cosd(engl_ang(3))];
                
% Throttle / Thrust relation
% Coeff_thrust and Coeff_throttle are set in the Jetcat P70 mask 
% initialization of the Engines subsystem in the 6DOF GTM model.
Coeff_thrust   = [0.000028864242464  -0.001914835880159 ...
    0.105916926095459  -1.251646340183624 ];
Coeff_throttle = [0.000033035538475  -0.008354385314499 ...
    1.191126728328792 31.518959847101669];
RPMref = Coeff_throttle*[dthr^3; dthr^2; dthr; 1];
Tdthr = Coeff_thrust*[RPMref^3; RPMref^2; RPMref;1];
Tdthr = pratio*Tdthr;

% Thrust model is in lbs. Convert to Newtons.
lbs2N      = 4.44822162;    % pound force to Newton [ N/lbs ]
Tdthr = Tdthr*lbs2N;      

% Engine Thrust: Forces and  Moments
neng = 2;
Tx = neng*DCV_LEng(1)*Tdthr;
Tz = neng*DCV_LEng(3)*Tdthr;
Tm = (Tx*deltaLEng(3)-Tz*deltaLEng(1));

% ----- Aero models        

% Dynamic pressure
qbar    = 0.5*rho0*V^2;

% Basic table--evaluate at beta=0 and current value of alpha
tbldata = Aero.C6_bas.data;
tbla = Aero.C6_bas.alpha;
tblb = Aero.C6_bas.beta;
idx = find(tblb==0);
alu = max( min(alpha*r2d,tbla(end)) , tbla(1) );

Cxa = interp1(tbla,tbldata(:,idx,1),alu);
Cza = interp1(tbla,tbldata(:,idx,3),alu);
Cma = interp1(tbla,tbldata(:,idx,5),alu);

% Control surface terms:
tbldata = Aero.dC3_ele.data;
tbla = Aero.dC3_ele.alpha;
tblb = Aero.dC3_ele.beta;
bidx = find(tblb==0);
tbls = Aero.dC3_ele.stab;
sidx = find(tbls==0);
tble = Aero.dC3_ele.elev;

alu = max( min(alpha*r2d,tbla(end)) , tbla(1) );
delu = max( min(de*r2d,tble(end)) , tble(1) );
tbldata = squeeze(tbldata(:,bidx,sidx,:,:));

Cxde = interp2(tbla,tble,tbldata(:,:,1)',alu,delu); 
Czde = interp2(tbla,tble,tbldata(:,:,2)',alu,delu);
Cmde = interp2(tbla,tble,tbldata(:,:,3)',alu,delu);

% Dynamic terms:
tbldata = Aero.dC3_q.data;
tbla = Aero.dC3_q.alpha;
tblq = Aero.dC3_q.qhat;
qhat = cbar*q/2/V;       % Normalized pitch rate [Dimensionless]

alu = max( min(alpha*r2d,tbla(end)) , tbla(1) );
qlu = max( min(qhat,tblq(end)) , tblq(1) );

Cxq = interp2(tbla,tblq,tbldata(:,:,1)',alu,qlu);
Czq = interp2(tbla,tblq,tbldata(:,:,2)',alu,qlu);
Cmq = interp2(tbla,tblq,tbldata(:,:,3)',alu,qlu);

% Combine terms
Cx = Cxa + Cxde + Cxq;
Cz = Cza + Czde + Czq;
Cm = Cma + Cmde + Cmq;

% ----- Forces / Moments
Fx = qbar*Sref*Cx + Tx - mass*g*sin(theta);
Fz = qbar*Sref*Cz + Tz + mass*g*cos(theta);
My = qbar*Sref*cbar*Cm + Tm;

% ----- State Equations
Vdot = ( Fx*cos(alpha) + Fz*sin(alpha) )/mass;
alphadot = ( -Fx*sin(alpha) + Fz*cos(alpha) )/(mass*V) + q;
qdot = My/Iyy;
thetadot = q;
xdot = [Vdot; alphadot;  qdot; thetadot];

